package MT::Plugin::SKR::SwitchRebuildEachArchive;
# SwitchRebuildEachCategory - Switch rebuilding each archive of category/folder
#       Copyright (c) 2009 SKYARC System Co.,Ltd.
#       http://www.skyarc.co.jp/

use strict;
use MT 4;
use MT::Template;
use MT::TemplateMap;
use Data::Dumper;#DEBUG

use vars qw( $MYNAME $VERSION );
$MYNAME = 'SwitchRebuildEachArchive';
$VERSION = '0.01';

use base qw( MT::Plugin );
my $plugin = __PACKAGE__->new({
        id => $MYNAME,
        key => $MYNAME,
        name => $MYNAME,
        version => $VERSION,
        author_name => 'SKYARC System Co.,Ltd.',
        author_link => 'http://www.skyarc.co.jp/',
#        doc_link => 'http://www.mtcms.jp/',
        description => <<HTMLHEREDOC,
<__trans phrase="Switch rebuilding each archive of category/folder">
HTMLHEREDOC
#        l10n_class => $MYNAME. '::L10N',
});
MT->add_plugin( $plugin );

sub instance { $plugin; }

### Registry
sub init_registry {
    my $plugin = shift;
    $plugin->registry({
        callbacks => {
            'build_file_filter' => \&_cb_build_file_filter,
            'MT::App::CMS::template_source.edit_category' => \&_cb_src_edit_category,
            'MT::App::CMS::template_source.edit_folder' => \&_cb_src_edit_category,
            'MT::App::CMS::template_param.edit_category' => \&_cb_param_edit_category,
            'MT::App::CMS::template_param.edit_folder' => \&_cb_param_edit_category,
            'cms_post_save.category' => \&_cb_post_save_category,
            'cms_post_save.folder' => \&_cb_post_save_category,
        },
    });
}



### Callback - Switch whether rebuild
sub _cb_build_file_filter {
    my ($cb, %opt) = @_;

    my $category = $opt{Category}
        or return 1;

    my $setting = load_plugindata ($category->id)
        or return 1; # not yet configured
    $setting = join ',', @{$setting};

    my $tmplmap = $opt{TemplateMap}
        or return 1;
    my $tmplmap_id = $tmplmap->id;
    $setting =~ /\b$tmplmap_id\b/;
}



### Callback - template_source.edit_category/edit_folder
sub _cb_src_edit_category {
    my ($eh, $app, $tmpl) = @_;

    my $old = quotemeta(<<'HTMLHEREDOC');
<mt:setvarblock name="action_buttons">
HTMLHEREDOC
    my $new = <<'HTMLHEREDOC';
    <fieldset>
        <h3><__trans phrase="SwitchRebuildEachArchive"></h3>
<mtapp:setting
    id="SwitchRebuildEachArchive"
    label="<__trans phrase="Enable rebuilding">">
<TMPL_IF NAME=SREA_LOOP>
  <ul><TMPL_LOOP NAME=SREA_LOOP>
    <li><input type="checkbox" name="<TMPL_VAR NAME=ID ESCAPE=HTML>" value="1" <TMPL_IF NAME=CHECKED>checked="checked"</TMPL_IF> />
    <TMPL_VAR NAME=TMPL_NAME ESCAPE=HTML> (<TMPL_VAR NAME=TYPE ESCAPE=HTML> - <TMPL_VAR NAME=PATH ESCAPE=HTML>)</li>
  </TMPL_LOOP></ul>
<TMPL_ELSE>
  <__trans phrase="No category archive mapping to rebuild">
</TMPL_IF>
</mtapp:setting>
    </fieldset>
HTMLHEREDOC
    $$tmpl =~ s/($old)/$new$1/;
}

### Callback - template_param.edit_category/edit_folder
sub _cb_param_edit_category {
    my ($cb, $app, $param, $tmpl) = @_;

    my $setting = load_plugindata ($param->{id}) || [];
    $setting = join ',', @{$setting};

    my @srea_loop;
    my $iter = MT::TemplateMap->load_iter({
        blog_id => $param->{blog_id},
        archive_type => [ 'Category', 'Category-Yearly', 'Category-Monthly', 'Category-Weekly', 'Category-Daily' ],
        build_type => [ 1, 4 ],
    });
    while (my $tmplmap = $iter->()) {
        my $tmpl = MT::Template->load({ id => $tmplmap->template_id })
            or next;
        my $tmplmap_id = $tmplmap->id;
        push @srea_loop, {
            id => 'srea_'. $tmplmap_id,
            tmpl_name => $tmpl->name,
            type => $tmplmap->archive_type,
            path => $tmplmap->file_template,
            checked => $setting ? 1 ; $setting =~ /\b$tmplmap_id\b/,
        };
    }
    $param->{srea_loop} = \@srea_loop if scalar @srea_loop;
}

### Callback - When configuration saved
sub _cb_post_save_category {
    my ($cb, $app, $obj) = @_;

    my @srea_loop;
    my $iter = MT::TemplateMap->load_iter({
        blog_id => $obj->blog_id,
        archive_type => [ 'Category', 'Category-Yearly', 'Category-Monthly', 'Category-Weekly', 'Category-Daily' ],
        build_type => [ 1, 4 ],
    });
    while (my $tmplmap = $iter->()) {
        push @srea_loop, $tmplmap->id if $app->param ('srea_'. $tmplmap->id);
    }
    save_plugindata ($obj->id, \@srea_loop);
}

########################################################################
use MT::PluginData;

sub save_plugindata {
    my ($key, $data_ref) = @_;
    my $pd = MT::PluginData->load({ plugin => &instance->key, key=> $key });
    if (!$pd) {
        $pd = MT::PluginData->new;
        $pd->plugin( &instance->id );
        $pd->key( $key );
    }
    $pd->data( $data_ref );
    $pd->save;
}

sub load_plugindata {
    my ($key) = @_;
    my $pd = MT::PluginData->load({ plugin => &instance->key, key=> $key })
        or return undef;
    $pd->data;
}

1;
__END__